function PPcalc(varargin)  % version  26-March-2002
% Calculation the push-pull tube stage (transformer output).
% Copyright (C) 2001 by Eugene V. Karpov. The author assumes no legal 
% liability arising from the use of this program.
% E-mail ekar@next-power.net

global MU EX KG1 KP KVB VCT a b Ug Type fname Pfn Us K 
Type='';
MU=[]; EX=[]; KG1=[]; KP=[]; KVB=[]; Us=[]; 
Ipmax=[]; Ppmax=[]; Upmax=[]; Ugmax=[];
K=0.6;										%Load distribution factor K=Zs/Zp or (Ws/Wp)^2
Ugstep=-1;									%Default Ug step
VCT=0;										%Default value
Pfn=1;										%Default value for pentode
N=4096;										%Default N-point for Fourier transform
%Upstep=1V is const.


%Load data file
%%%%%%%%%%%%%%%%%%%%%%%% Format data file  %%%%%%%%%%%%%%%%%%%%%%%%%%%
%File name  ---> 6N8C.mod
%Name='6N8C'; Type='Triode'; or Pentode
% Model data:
%MU=22.87;  EX=1.516;  KG1=2209.8;  KP=167.87; KVB=155.4;  VCT=0.70; 
% Analysis data: 
%Ipmax=.02; Ppmax=2.7; Upmax=300; Ugmax=-10; Ugstep=-0.5;
%for pentode add following parameters:
%KG2=  ;  Us=   ;		K=		;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[fname,pname] = uigetfile('*.mod');	%Selection data file
if isempty(findstr(fname,'.mod'))
   error('File not selected');
end
fid=fopen(fname,'rt');					%Open data file
if fid<0
   error('Do not reading');
end
while 1  line = fgetl(fid);			%Load data file 
   if ~isstr(line)
      break 
   end
   eval(line,'fprintf(''%s%s\n'',12,lasterr);');
end; 
fclose(fid);
%Check data
errmes1='Shortage data, absence'; errmes2=' Check up data file.';
if isempty(MU); error([errmes1,' MU.',errmes2]); end
if isempty(KVB); error([errmes1,' KVB.',errmes2]); end
if isempty(EX); error([errmes1,' EX.',errmes2]); end
if isempty(KG1); error([errmes1,' KG1.',errmes2]); end
if isempty(KP); error([errmes1,' KP.',errmes2]); end
if isempty(Upmax); error([errmes1,' Upmax.',errmes2]); end
if isempty(Ipmax); error([errmes1,' Ipmax.',errmes2]); end
if isempty(Ugmax); error([errmes1,' Ugmax.',errmes2]); end
if isempty(Ppmax); error([errmes1,' Ppmax.',errmes2]); end

%Check tube type
switch Type;
     case 'Pentode',...
     if isempty(Us); error([errmes1,' Us.',errmes2]); end,...		%Check data
     if isempty(KG2); error([errmes1,' UG2.',errmes2]); end,...	%Check data
     case 'Triode',
  otherwise
     clear
         error('Absence tube Type')
end

%Setting
nstep=ceil(Ugmax./Ugstep);		%Step for Ug
cm=['k','b','g','r','m'];		%COLOR MAP	
cp=1;									%Color pointer
dat2=[''];
%-----------------------------------------
%Calculation Ri and S for triode
%Calculation plate carrent

if strcmp(Type,'Triode');
   %Triode
  while 1   
	h = waitbar(0,'Please wait...');
	for g1=0:1:nstep;
   	Ug=g1.*Ugstep;
   	for p=1:1:Upmax
       	Up=p;
       	I(g1+1,p)=inst(Up,Ug);
    	end
    waitbar(g1./nstep);
	end
	close(h);

	%Calculation the max plate current
	Upplot=1:1:Upmax; 
	Imaxplot=Ppmax./Upplot;

	clf reset;
	plot(Upplot,I,'b'); grid on; axis([0 Upmax 0 Ipmax]); xlabel('Up'); ylabel('Ip');...
	title(['ENTER A TEST LOAD LINE FOR TRIODE ',Name,]);
	hold on 
	plot(Imaxplot,'r');
	set(1,'Pointer','fullcrosshair');
	%Input the load line of a test 
	x=[]; y=[];
    while isempty(x)
       [x,y]=ginput(1);       %first point
    end   
    p=length(x); g1=length(y);
    Ip1=y(g1); Up1=x(p);
    plot(Up1,Ip1,'rx','MarkerSize',15);
    
    x=[]; y=[];
    while isempty(x)
       [x,y]=ginput(1);       %Second point
    end
    p=length(x); g1=length(y);
    Ip2=y(g1); Up2=x(p);
    if Ip2>=Ip1;				%Check Ip2 value --> Ip2>Ip1
    	Ip2=Ip1.*.99;
    end
    plot(Up2,Ip2,'rx','MarkerSize',15);
    il=(((Up1-Upplot).*(Ip2-Ip1))./(Up1-Up2))+Ip1;
    a=(-(Up1.*Ip2-Up2.*Ip1))./(Ip1-Ip2);
	 b=((Up1.*(Ip2-Ip1))./(Up1-Up2))+Ip1;
    Rl=a./il(1);
    plot(il,'r')				%Plot load line	
	%Calculation Ri 
	h = waitbar(0,'Please wait...');
	for g1=0:1:nstep;
   	Ug=g1.*Ugstep;
   	x=fminsearch('se',[20],[]);
   	sUp(g1+1)=x;
   	Up=x;
      il1(g1+1)=inst(Up,Ug);		%Test plate current
	end
	sUp=sUp+1;							%Delta Up=1V
	for g1=0:1:nstep
   	Ug=g1.*Ugstep;
   	Up=sUp(g1+1);
   	il2(g1+1)=inst(Up,Ug);		%Delta I plate
   	waitbar(g1+1./nstep)
	end
%----------------------------------------- modify
	Ir=(il2-il1);
	Ir=Ir.*(Ir>.1e-6);
	Ir=rot90(nonzeros(Ir));
	ri=1./Ir;
   Ir=il1(1:size(Ir,2));
   Ri=mean(ri);						%Ohm
   Rtube=sprintf('%6.1f',Ri);
	%Plot data
	figure(1);
	subplot(2,1,1);plot(Ir,ri); grid on; axis tight;...
	xlabel('Ip'); ylabel('Ri'); title([Name,' Output resistans  Ri=',Rtube,' Ohm']);
	clear il2;
	close(h);
	%Calculation S
	sUp=sUp-1;
	for g1=0:1:nstep
   	Ug=g1.*Ugstep+Ugstep./5;
   	Up=sUp(g1+1);
   	il2(g1+1)=inst(Up,Ug);		%Delta I plate
	end
	il3=(il2-il1);
	s=il3./(Ugstep./5);
   S=mean(s);
   Stube=sprintf('%3.1f',S.*1000);
	%Plot data
	figure(1);
	subplot(2,1,2); plot(il1,s); grid on; axis tight; title([Name,' Transconductance  S=',...
   Stube,' mA/V']); xlabel('Ip'); ylabel('S');
	zoom on;
   set(1,'Pointer','circle');
   select=menu('Your choice','New test load',...  
   'Quit test');
	if  select==1;
      clear il1;
      clear il2;
      close all;
   else  break; end;
   end
   zoom off;
   else
    %Pentode -> not calculation
    Ri=5000;
end

%-----------------------------------------
%Input the work load line
%Calculation plate current
h = waitbar(0,'Please wait...');
for g1=0:1:nstep;
   Ug=g1.*Ugstep;
   for p=1:1:Upmax
       Up=p;
       I(g1+1,p)=inst(Up,Ug);
    end
    waitbar(g1./nstep);
end
close(h);

%Calculation max plate current
Upplot=1:1:Upmax; 
Imaxplot=Ppmax./Upplot;

clf reset;
plot(Upplot,I,'b'); grid on; axis([0 Upmax 0 Ipmax]); xlabel('Up'); ylabel('Ip');...
title([Name,' PLATE CHARACTERISTICS']);
hold on 
plot(Imaxplot,'r');
set(1,'Pointer','fullcrosshair');

while 1;
   if cp==5;						%set color plot
      cp=0;
   end
   cp=cp+1;
c1=1;

%Entry load line
while c1==1;
select=menu('Entry load line','Draw load line by Rl and Usup',...
   'Draw load line by two point');    %Select next step
if isempty(select);
   	close all;
   	clear
    	error('Break-down Calculation')
    elseif select==1;
       while 1
  		 	 def={sprintf('%6.1f',Ri),sprintf('%3.1f',Upmax)};
          lines=1;
          prompt={'R load Ohm','U supply V'}; titlle='Enter data';...
       	 select=inputdlg(prompt,titlle,lines,def);
          if  isempty(select);
          else;  break;
       	 end
        end
        if isempty(select{1});					%Check data
           select{1}=sprintf('%6.1f',Ri);		   
        end
        if isempty(select{2});					%Check data
           select{2}=sprintf('%3.1f',Upmax);
        end
        
        Rl=eval(select{1}); Usup=eval(select{2}); %Calculation load line by Rl and Usup
        %For transformer output
        %Rl=Rl./4;
        %it is 1/4 the plate to plate load impedance
    	  il=(-1./Rl).*Upplot+Usup./Rl;
        a=Usup; b=Usup./Rl;
        plot(il,cm(cp))										%Plot load line
 else select==2;			%Calculation load line by two point
    x=[]; y=[];
    while isempty(x)
       [x,y]=ginput(1);       %first point
    end   
    p=length(x); g1=length(y);
    Ip1=y(g1); Up1=x(p);
    plot(Up1,Ip1,'rx','MarkerSize',15);
    
    x=[]; y=[];
    while isempty(x)
       [x,y]=ginput(1);       %Second point
    end
    p=length(x); g1=length(y);
    Ip2=y(g1); Up2=x(p);
    if Ip2>=Ip1;				%Check Ip2 value --> Ip2>Ip1
       Ip2=Ip1.*.99;
    end
    plot(Up2,Ip2,'rx','MarkerSize',15);
    il=(((Up1-Upplot).*(Ip2-Ip1))./(Up1-Up2))+Ip1;
    a=(-(Up1.*Ip2-Up2.*Ip1))./(Ip1-Ip2);
	 b=((Up1.*(Ip2-Ip1))./(Up1-Up2))+Ip1;
    Rl=a./il(1);
    plot(il,cm(cp))		%Plot load line	
 end
  select=menu('Your choice','New R load',...  %Select next step
  'CONTINUE');
if isempty(select);
   close all;
   clear
   error('Break-down Calculation')
elseif select==2;
       c1=2;
    end
end
while c1==2;
 %Draw Load line - il1=f(Ug)
    h = waitbar(0,'Please wait...');
for g1=0:1:nstep*10;
   Ug=g1.*Ugstep/10;    
   sUg(g1+1)=Ug;   			   %Step Ug
   x=fminsearch('se',[20],[]);
   Up=x;
   il1(g1+1)=inst(Up,Ug); 		% Plate current
   waitbar(g1./nstep./10);
end
close(h);
figure(2);
plot(sUg,il1,cm(cp)); grid on; xlabel('Ug'); ylabel('Ip');...
   title([Name,' GRID-PLATE CHARACTERISTICS']);
%zoom on

%Calculation stage parameters
x=[]; y=[];
while isempty(x);										% Input Ug0 (quiescent state)
   [x,y]=ginput(1);
end
p=length(x); g1=length(y);
Ug0=x(p);
hold on;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%For PP stage
Ug0=ceil(Ug0./(Ugstep./10)).*(Ugstep./10);

Ug=Ug0; x=fminsearch('se',[20],[]); Up0=x; Up=x; I0=inst(Up,Ug);
plot(Ug0,I0,'rx','MarkerSize',15);
hold off;

UgPP=Ugmax.*2-(Ugmax-Ug0);							%New length data
nstepPP=ceil(UgPP./Ugstep);						%News step for Ug
%Draw Load line for PP stage
    h = waitbar(0,'Please wait...');
for g1=0:1:nstepPP*10;
   Ug=g1.*Ugstep/10;    
   sUg(g1+1)=Ug;   			   					%Step Ug
   x=fminsearch('se',[20],[]);
   Up=x;
   Ul1(g1+1)=Up;
   il1(g1+1)=inst(Up,Ug); 							%Plate current
   waitbar(g1./nstepPP./10);
end
close(h);
					
il2=fliplr(il1.*-1);									%Rotate characteristics
Index1=ceil(Ug0/Ugstep.*10+1);
Index2=ceil((size(il2,2)-Index1)+1);
Index3=Index1.*2-1;
il1=zeros(1,Index3)+il1(1,1:Index3);
Ul1=zeros(1,Index3)+Ul1(1,1:Index3);
il2=zeros(1,Index3)+il2(1,Index2-Index1+1:size(il2,2));

Imid=trapz(il1)./(Index3-1);						%Middle current
Pplate=trapz(il1.*Ul1)./(Index3-1);				%Plate power

il12=il1+il2;											%Composite Load Line
sUg=linspace(0,2.*Ug0,Index3);
IH=spline(sUg,il12,Ug0/2);							%Current for Ug0/2
IH1=(2.*(il12(1)+IH))./3;							%First harmonic current
IH3=(il12(1)-2.*IH)./3;								%Third harmonic current
P=(Rl.*IH1.^2)./2;									%Output power

figure(2);												%Plot data

plot(sUg,il1,cm(cp)); grid on; xlabel('Ug'); ylabel('Ip');...
   title([Name,' GRID-PLATE Composite Curve']);
hold on;
plot(Ug0,I0,'rx','MarkerSize',10);
plot(Ug0,I0.*-1,'rx','MarkerSize',10);
plot(sUg,il2,cm(cp));
plot(sUg,il12,cm(cp));axis tight;
hold off;

dUp=Rl.*IH1;					%Output voltage
Kg=dUp./Ug0;					%Gain
P0=I0.*a;						%Quiescent current
Psup=Imid.*a.*2;				%Power supply
H3=IH3/IH1.*100;				%Third harmonic

clear il1; clear il2; clear Index1;
clear Index2; clear Index3; clear Ul1;

%Print data
if isempty(dat2)
   disp(sprintf('\n'));
   disp(['PP-stage   ','           ',date]);
	disp(['Tube  ',Name,'    ',Type,' Mode']);
else
   disp('New set');
end
disp(['Third harmonic=',sprintf('%2.2f',H3),'%']);
disp(['Pout=',sprintf('%2.2f',P),'W','  Uout=',sprintf('%3.1f',dUp),'V',...
      '  G=',sprintf('%3.1f',Kg),'  Rload=',sprintf('%3.1f',Rl),'Ohm']);
disp(['Ug0=',sprintf('%2.2f',Ug0),'V','  I0=',sprintf('%3.4f',I0),'A',...
      '  P0=',sprintf('%3.1f',P0),'W','   Pplate=',sprintf('%3.1f',Pplate),'W']);
disp(['Usup=',sprintf('%4.1f',a),'V','   Psup=',sprintf('%3.1f',Psup),'W']);

dat1=['   H3=',sprintf('%2.2f',H3),'%','  Pout=',sprintf('%2.2f',P),'W',...
      '   P0=',sprintf('%3.1f',P0),'W','   Pplate=',sprintf('%3.1f',Pplate),'W',...
      '   Uout=',sprintf('%3.1f',dUp),'V','  G=',sprintf('%3.1f',Kg),...
	   '   Rl=',sprintf('%6.1f',Rl./1000),'KOhm'];
h=outData(dat1,dat2);
dat2=dat1;

%Select next step
c1=5;
while c1==5;

select=menu('Spectrum','           Yes          ','No'); 
if isempty(select);
    close all
  clear
  error('Break-down Calculation')
   elseif select==1;
   %Calculation spectrum
   %Load max magnitude Ug
	prompt={'V'}; titlle='Input grid magnitude';
	def={sprintf('%2.1f',abs(Ug0))};
	lines=1;
   select=inputdlg(prompt,titlle,lines,def);
   if isempty(select); break; end;
   if isempty(select{1}); break; end;        
	UgmaxS=abs(eval(select{1}));				%Magnitude test signal
	if UgmaxS>abs(Ug0);
   	UgmaxS=abs(Ug0);
	end

	t=0:1*10^-3/(N-1):10^-3;					%Generation test signal f=1kHz
   UgS=UgmaxS.*sin([2.*pi.*1000*t]);
   UgS=UgS+Ug0;
	
   %Calculation output voltage
   IS=spline(sUg,il12,UgS);
   	
	%Fourier transform
	Y=fft(IS,N);								
   SS=abs(Y)./(N./2);							%Harmonics magnitude
   
   dUp=Rl.*SS(2);									%Output voltage
   P=(Rl.*SS(2).^2)./2;							%Output power
   
	D=SS(3:12).^2;									%Total Harmonic Distortion
	D=sqrt(sum(D))./SS(2).*100;				
	SS=SS(1:13)./SS(2).*100;					%Normalization
	SS(2)=0;											%Delete first harmonic
	f=1000*(0:12);									%Plot data
	figure(4);
	stem(f,SS,'filled'); grid on;	xlabel('kHz'); ylabel('%');...
      title([Name,' Spectrum output voltage. Magnitude Ug=',sprintf('%3.1f',UgmaxS)]);
   zoom on;
   set(4,'Pointer','circle');
   %Print result
   disp('Calculation spectrum:');
	disp(['Magnitude Ug=',sprintf('%3.1f',UgmaxS),'V']);
	disp(['Pout=',sprintf('%2.2f',P),'W','   Magnitude Uout=',sprintf('%3.1f',dUp),'V']);
	disp(['HD=',sprintf('%2.2f',D),'%','  HD2=',sprintf('%2.2f',SS(3)),'%',...
      		'  HD3=',sprintf('%2.2f',SS(4)),'%','  HD4=',sprintf('%2.3f',SS(5)),'%',...
            '  HD5=',sprintf('%2.3f',SS(6)),'%']);
   dat1=['HD=',sprintf('%2.2f',D),'%','  HD2=',sprintf('%2.2f',SS(3)),'%',...
      		'  HD3=',sprintf('%2.2f',SS(4)),'%','  HD4=',sprintf('%2.3f',SS(5)),'%',...
            '  HD5=',sprintf('%2.3f',SS(6)),'%','    Pout=',sprintf('%2.2f',P),'W',...
            '  Uout=',sprintf('%3.1f',dUp),'V'];
      close(h);  h=outData(dat1,dat2); dat2=dat1;
   else
      c1=10;
 end  
end
close(figure(4)); zoom off;
 
%Select next step
select=menu('Your choice','New Ug',...  
   'New R load','Quit');
if  select==1; 
   c1=2;
   figure(2);
   close(h);
   clear il12;
   clear sUg;
elseif select==2;
   c1=3;
   figure(1);
   close(h);
   clear il12;
   clear sUg;
  else 
   close all
   clear
  return
end
end
end












   
   




   


